package jp.littleforest.util;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 正規表現に関するユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class RegexUtil {
    private static final Map<String, Pattern> patternCache = new HashMap<String, Pattern>();

    /**
     * 指定された文字列から正規表現の <i>n</i> 番目の先方参照に一致する文字列を返します。<br />
     * 
     * @param text
     *        文字列
     * @param regex
     *        正規表現
     * @param index
     *        取り出す先方参照の位置。{@link Matcher#group(int)} に準ずる
     * @param flags
     *        正規表現フラグ
     * @return 先方参照に一致する文字列の配列。一致しない場合は <code>null</code>
     */
    public static String parseText(final String text, final String regex,
            final int index, final int flags) {
        Pattern pattern = getPattern(regex, flags);
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            return matcher.group(index);
        } else {
            return null;
        }
    }

    /**
     * 指定された文字列から正規表現の <i>n</i> 番目の先方参照に一致する文字列を返します。<br />
     * 
     * @param text
     *        文字列
     * @param regex
     *        正規表現
     * @param index
     *        取り出す先方参照の位置。{@link Matcher#group(int)} に準ずる
     * @return 先方参照に一致する文字列の配列。一致しない場合は <code>null</code>
     */
    public static String parseText(final String text, final String regex,
            final int index) {
        return parseText(text, regex, index, 0);
    }

    /**
     * 指定された文字列から正規表現の先方参照に一致する文字列を取得します。<br />
     * 
     * @param text
     *        文字列
     * @param regex
     *        正規表現
     * @param flags
     *        正規表現フラグ
     * @return 先方参照に一致する文字列の配列。一致しない場合は <code>null</code>
     */
    public static String[] parseTexts(final String text, final String regex,
            final int flags) {
        Pattern pattern = getPattern(regex, flags);
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            int count = matcher.groupCount();
            String[] result = new String[count];
            for (int i = 0; i < count; i++) {
                result[i] = matcher.group(i + 1);
            }
            return result;
        } else {
            return null;
        }
    }

    /**
     * 指定された文字列から正規表現の先方参照に一致する文字列を取得します。<br />
     * 
     * @param text
     *        文字列
     * @param regex
     *        正規表現
     * @return 先方参照に一致する文字列の配列。一致しない場合は <code>null</code>
     */
    public static String[] parseTexts(final String text, final String regex) {
        return parseTexts(text, regex, 0);
    }

    /**
     * 指定されたフラグを使用して、指定された正規表現をパターンにコンパイルします。<br />
     * コンパイルしたパターンはキャッシュされます。<br />
     * 
     * @param regex
     *        正規表現
     * @param flags
     *        マッチフラグ。{@link Pattern#CASE_INSENSITIVE}、 {@link Pattern#MULTILINE}
     *        、{@link Pattern#DOTALL}、 {@link Pattern#UNICODE_CASE}、
     *        {@link Pattern#CANON_EQ}を含むビットマスク。
     * @return パターン
     */
    public static Pattern getPattern(final String regex, final int flags) {
        String key = regex + flags;
        Pattern pattern = patternCache.get(key);
        if (pattern != null) {
            return pattern;
        } else {
            pattern = Pattern.compile(regex, flags);
            patternCache.put(key, pattern);
            return pattern;
        }
    }

    /**
     * 指定された正規表現をパターンにコンパイルします。<br />
     * コンパイルしたパターンはキャッシュされます。<br />
     * 
     * @param regex
     *        正規表現
     * @return パターン
     */
    public static Pattern getPattern(final String regex) {
        return getPattern(regex, 0);
    }

}
