/**
 * Copyright 2009 Yusuke Komori.
 */
package jp.littleforest.util;

/**
 * スタックトレースに関するユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class StackTraceUtil {
    private StackTraceUtil() {

    }

    /**
     * 現在実行しているメソッドの名称を取得します。<br />
     * 
     * @return 実行しているメソッドの名称。何らかの理由で取得できない場合は空文字列。
     */
    public static String getExecutingMethodName() {
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        // stackTrace[0] は getStackTrace() への呼び出しを表し、
        // stackTrace[1] は getExecutingMethodName() への呼び出しを表す
        if (stackTrace.length >= 2) {
            return stackTrace[2].getMethodName();
        } else {
            return "";
        }
    }

    /**
     * 現在実行しているメソッドの名称を取得します。<br />
     * メソッド名称を取得したいメソッドから直接本メソッドを呼び出していない場合、 level 引数でスタックトレースの階層を補正できます。
     * 
     * <pre>
     * foo() -&gt; bar() -&gt; StackTraceUtil.getExecutingMethodName()
     * </pre>
     * 
     * たとえば、上記のような例では通常メソッド名として "bar" を返しますが、 level に 1を指定することで "foo"
     * を得ることができます。
     * 
     * @param level メソッド名称を取得するスタックトレースのレベル(0以上)
     * @return 実行しているメソッドの名称。何らかの理由で取得できない場合は空文字列。
     */
    public static String getExecutingMethodName(final int level) {
        AssertionUtil.assertZeroOrMore("level", level);

        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        int idx = 2 + level;
        try {
            return stackTrace[idx].getMethodName();
        } catch (ArrayIndexOutOfBoundsException ex) {
            return "";
        }
    }
}
