package jp.littleforest.util.eval;

import java.lang.reflect.Method;

/**
 * パフォーマンス測定に関するユーティリティクラスです。 <br />
 * 
 * @author y-komori
 */
public class EvalUtil {
    private EvalUtil() {

    }

    /**
     * 指定された名前の {@link Method} オブジェクトを取得します。<br />
     * 
     * 同一名称のメソッドが複数存在する場合、最初に見つかったメソッドを返します。また、親クラスは検索しません。
     * 
     * @param clazz
     *            メソッドを探す {@link Class} オブジェクト
     * @param methodName
     *            メソッド名
     * @return 見つかった {@link Method} オブジェクト。見つからない場合は <code>null</code>。
     */
    public static Method getMethod(final Class<?> clazz, final String methodName) {
        Method[] methods = clazz.getDeclaredMethods();
        for (Method method : methods) {
            if (method.getName().equals(methodName)) {
                return method;
            }
        }
        return null;
    }

    /**
     * 実行時間を表示するユーティリティメソッドです。<br />
     * 
     * @param startTime
     *            開始時刻
     * @param endTime
     *            終了時刻
     * @param count
     *            実行回数
     */
    public static void printResult(final long startTime, final long endTime,
            final int count) {
        long msec = endTime - startTime;
        double perSec = (double) (1000 * count) / (double) msec;
        System.out.printf("実行時間 : %,6d msec (%,.2f 件/sec)\n", msec, perSec);
    }

    /**
     * メソッドの実行時間を計測します。<br />
     * 
     * メソッドが int または long 値を返す場合、その値をメソッドの実行時間として集計します。<br />
     * 
     * @param iteration
     *            計測回数
     * @param count
     *            1回の計測あたりの実行カウント
     * @param method
     *            計測対象メソッド
     * @param object
     *            実行対象オブジェクト
     * @param args
     *            メソッドに対する引数
     * @throws Exception
     */
    public static void evalAverageTime(final int iteration, final int count,
            final Method method, final Object object, final Object... args)
            throws Exception {
        long result = 0;
        for (int i = 1; i <= iteration; i++) {
            System.out.printf("--- %4d 回目テスト開始 ---  ", i);
            long startTime = System.currentTimeMillis();
            Object methodResult = method.invoke(object, args);
            long endTime = System.currentTimeMillis();
            if (methodResult instanceof Integer) {
                result += ((Integer) methodResult).intValue();
            } else if (methodResult instanceof Long) {
                result += ((Long) methodResult).longValue();
            } else {
                result += endTime - startTime;
            }
        }
        double average = (double) result / (double) iteration;
        double averagePerSec = (count * 1000) / average;
        System.out.printf("平均実行時間 : %,6.2f msec (%,.2f 件/sec)\n", average,
                averagePerSec);

    }
}
