package jp.littleforest.util.html;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import au.id.jericho.lib.html.Attribute;
import au.id.jericho.lib.html.Attributes;
import au.id.jericho.lib.html.Element;
import au.id.jericho.lib.html.EndTag;
import au.id.jericho.lib.html.Segment;
import au.id.jericho.lib.html.Source;
import au.id.jericho.lib.html.StartTag;
import au.id.jericho.lib.html.Tag;

/**
 * HTML を扱うためのユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class HtmlUtil {
    private HtmlUtil() {

    }

    /**
     * HTML から {@link Source} オブジェクトを生成します。<br />
     * 
     * @param html
     *        HTML
     * @return {@link Source} オブジェクト
     */
    public static Source createSource(final String html) {
        return new Source(html);
    }

    /**
     * 要素の中から指定された <code>class</code> 属性をもつ要素を返します。<br />
     * 要素が複数存在する場合、最初の要素を返します。<br />
     * 
     * @param segment
     *        検索対象セグメント
     * @param className
     *        <code>class</code> 属性名
     * @return 見つかった要素。見つからなかった場合は <code>null</code>。
     */
    public static Element findElementByClass(final Segment segment,
            final String className) {
        return findElementByClass(segment, className, 0);
    }

    /**
     * 要素の中から指定された <code>class</code> 属性をもつ要素を返します。<br />
     * 要素が複数存在する場合、<code>index</code> で指定された要素を返します。<br />
     * 
     * @param segment
     *        検索対象セグメント
     * @param className
     *        <code>class</code> 属性名
     * @param index
     *        取り出す要素のインデックス(最初の要素は 0)
     * @return 見つかった要素。見つからなかった場合は <code>null</code>。
     */
    @SuppressWarnings("unchecked")
    public static Element findElementByClass(final Segment segment,
            final String className, final int index) {
        List<Element> elements = segment.findAllElements("class", className,
                true);
        if (elements != null && elements.size() > index) {
            return elements.get(index);
        } else {
            return null;
        }
    }

    /**
     * 要素の中から指定された名前をもつ要素を返します。<br />
     * 要素が複数存在する場合、最初の要素を返します。
     * 
     * @param segment
     *        検索対象要素
     * @param elementName
     *        要素名
     * @return 見つかった要素。見つからなかった場合は <code>null</code>。
     */
    public static Element findFirstElement(final Segment segment,
            final String elementName) {
        List<Element> elements = findAllElements(segment, elementName);
        if (elements != null && elements.size() > 0) {
            return elements.get(0);
        } else {
            return null;
        }
    }

    /**
     * 要素が持つテキストを取り出します。<br />
     * 子要素の持つテキストは無視します。また、子要素に不正なタグが合った場合も無視します。<br />
     * 
     * @param segment
     *        対象セグメント
     * @return 取り出したテキスト。<code>segment</code> が <code>null</code> の場合は
     *         <code>null</code>
     */
    public static String extractText(final Segment segment) {
        if (segment == null) {
            return null;
        }

        StringBuffer result = new StringBuffer(128);
        Stack<Segment> segStack = new Stack<Segment>();
        Iterator<Segment> itr = getNodeIterator(segment);
        while (itr.hasNext()) {
            Segment seg = itr.next();
            if (seg instanceof StartTag) {
                Element element = ((StartTag) seg).getElement();
                if (element.getEndTag() != null) {
                    segStack.push(seg);
                }
            } else if (seg instanceof EndTag) {
                Element element = ((EndTag) seg).getElement();
                if (element != null && !segStack.empty()) {
                    segStack.pop();
                }
            } else {
                if (segStack.size() == 1) {
                    result.append(seg.getTextExtractor().toString());
                }
            }

        }

        return result.toString();
    }

    /**
     * 指定された文字列からタグ部分を取り除いた文字列を返します。<br />
     * 
     * @param html
     *        文字列
     * @return タグ部分を取り除いた文字列
     */
    public static String eliminateTags(final String html) {
        Source source = createSource(html);
        return source.getTextExtractor().toString();
    }

    /**
     * 指定された名前の要素をすべて取り出します。<br />
     * 
     * @param segment
     *        セグメント
     * @param tag
     *        要素名
     * @return 要素のリスト
     * @see Tag
     */
    @SuppressWarnings("unchecked")
    public static List<Element> findAllElements(final Segment segment,
            final String tag) {
        return segment.findAllElements(tag);
    }

    /**
     * {@link Segment} から {@link Iterator} を取得します。<br />
     * 
     * @param segment
     *        対象 {@link Segment} オブジェクト
     * @return {@link Iterator} オブジェクト
     */
    @SuppressWarnings("unchecked")
    public static Iterator<Segment> getNodeIterator(final Segment segment) {
        return segment.getNodeIterator();
    }

    /**
     * 指定された {@link Element} の子要素のリストを返します。<br />
     * 
     * @param element
     *        {@link Element} オブジェクト
     * @return 子要素のリスト
     */
    @SuppressWarnings("unchecked")
    public static List<Element> getChildElements(final Element element) {
        return element.getChildElements();
    }

    /**
     * HTML を指定されたマーカ文字列で分離します。<br />
     * 
     * @param html
     *        HTML
     * @param marker
     *        マーカ文字列
     * @return 分離したHTMLのリスト
     */
    @Deprecated
    public static List<String> splitHtml(final String html, final String marker) {
        ArrayList<String> parts = new ArrayList<String>();
        int prevPos = 0;

        for (int pos = 0; (pos = html.indexOf(marker, pos)) != -1; pos += marker
                .length(), prevPos = pos) {
            parts.add(html.substring(prevPos, pos));
        }
        parts.add(html.substring(prevPos, html.length()));

        return parts;
    }

    /**
     * {@link Source} の内容を {@link Writer} へ出力します。<br />
     * 
     * @param writer
     *        出力先
     * @param source
     *        出力対象
     * @param indent
     *        インデント文字列
     * @throws IOException
     *         入出力エラーが発生した場合
     */
    public static void viewSource(final Writer writer, final Source source,
            final String indent) throws IOException {
        int pos = 0;
        Element element;
        while ((element = source.findNextElement(pos)) != null) {
            viewElement(writer, element, indent, "");
            pos = element.getEnd();
        }
        writer.flush();
    }

    /**
     * {@link Element} の内容を {@link Writer} へ出力します。<br />
     * 
     * @param writer
     *        出力先
     * @param element
     *        出力対象
     * @param indent
     *        インデント文字列
     * @throws IOException
     *         入出力エラーが発生した場合
     */
    public static void viewElement(final Writer writer, final Element element,
            final String indent) throws IOException {
        viewElement(writer, element, indent, "");
    }

    private static void viewElement(final Writer writer, final Element element,
            final String indent, final String currentIndent) throws IOException {
        writer.write(currentIndent + "<" + element.getName());
        viewAttribute(writer, element);
        writer.write(">\n");

        List<Element> children = getChildElements(element);
        for (Element child : children) {
            viewElement(writer, child, indent, currentIndent + indent);
        }
        String text = extractText(element);
        if (text != null && !"".equals(text)) {
            writer.write(currentIndent + indent + text + "\n");
        }

        writer.write(currentIndent + "</" + element.getName() + ">\n");
    }

    private static void viewAttribute(final Writer writer, final Element element)
            throws IOException {
        Attributes attributes = element.getAttributes();
        if (attributes == null) {
            return;
        }
        Iterator<?> itr = attributes.iterator();
        while (itr.hasNext()) {
            Attribute attribute = (Attribute) itr.next();
            writer.write(" " + attribute.getName() + "=\""
                    + attribute.getValue() + "\"");
        }
    }

    /**
     * 文字列に対して HTML エスケープを行います。<br />
     * 具体的には、以下の変換を行います。<br />
     * 
     * <pre>
     *   改行 → &lt;br /&gt;
     *   &lt; → &amp;lt;
     *   &gt; → &amp;gt;
     *   &amp; → &amp;amp;
     *   &quot; → &amp;quot;
     * </pre>
     * 
     * @param str
     *        対象文字列
     * @return エスケープされた文字列
     */
    public static String escapeHtml(final String str) {
        if (str == null) {
            return null;
        }

        int size = str.length();
        StringBuilder buf = new StringBuilder(size);
        for (int i = 0; i < size; i++) {
            char ch = str.charAt(i);
            switch (ch) {
            case '\n':
                buf.append("<br />\n");
                break;
            case '<':
                buf.append("&lt;");
                break;
            case '>':
                buf.append("&gt;");
                break;
            case '&':
                buf.append("&amp;");
                break;
            case '"':
                buf.append("&quot;");
                break;
            default:
                buf.append(ch);
                break;
            }
        }
        return buf.toString();
    }
}
