package jp.littleforest.util.html.query;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import au.id.jericho.lib.html.Element;

/**
 * 要素を取得するためのコマンドです。<br />
 * 要素名[@属性名='属性値'][順番] にマッチします
 * 
 * @author y-komori
 */
public class ElementCommand implements QueryCommand {
    private static final String REGEX = "([A-Za-z0-9_]+)?(\\[@([A-Za-z]+)='([^']*)'\\])?(\\[([0-9])+\\])?";

    private Pattern pattern;

    public ElementCommand() {
        this.pattern = Pattern.compile(REGEX);
    }

    public boolean matches(final Object target, final String command) {
        if (target instanceof Element || target instanceof List) {
            Matcher matcher = pattern.matcher(command);
            return matcher.matches();
        } else {
            return false;
        }
    }

    public Object process(final Object target, final String command) {
        Matcher matcher = pattern.matcher(command);
        matcher.find();

        String tag = matcher.group(1);
        String attrName = matcher.group(3);
        String attrValue = matcher.group(4);
        String index = matcher.group(6);

        List<Element> targetList = getTargetList(target);
        if (targetList == null) {
            return null;
        }

        if (tag != null) {
            // 要素名が指定されている場合
            if (attrName != null) {
                // 属性が指定されている場合
                List<Element> elements = findElements(targetList, tag,
                        attrName, attrValue);
                return getElementAt(normalizeIndex(index), elements);
            } else {
                // 属性が指定されていない場合
                List<Element> elements = findElements(targetList, tag);
                return getElementAt(normalizeIndex(index), elements);
            }
        } else {
            // 要素名が指定されていない場合
            if (attrName != null) {
                // 属性名が指定されている場合
                List<Element> elements = findElements(targetList, attrName,
                        attrValue);
                return getElementAt(normalizeIndex(index), elements);
            } else {
                // 属性名が指定されていない場合
                return getElementAt(normalizeIndex(index), targetList);
            }
        }
    }

    private int normalizeIndex(final String indexStr) {
        // 順番が未指定の場合は -1 を返す
        int idx = -1;
        if (indexStr != null) {
            idx = Integer.parseInt(indexStr) - 1;
            idx = idx < 0 ? 0 : idx;
        }
        return idx;
    }

    private Object getElementAt(final int idx, final List<Element> elements) {
        if (idx >= 0) {
            if (idx < elements.size()) {
                return elements.get(idx);
            } else {
                return null;
            }
        } else {
            return elements;
        }
    }

    @SuppressWarnings("unchecked")
    private List<Element> getTargetList(final Object target) {
        if (target instanceof Element) {
            return ((Element) target).getChildElements();
        } else if (target instanceof List) {
            List<Element> targetList = new ArrayList<Element>();
            for (Element element : (List<Element>) target) {
                targetList.addAll(element.getChildElements());
            }
            return targetList;
        } else {
            return null;
        }
    }

    private List<Element> findElements(final List<Element> target,
            final String attrName, final String attrValue) {
        List<Element> result = new ArrayList<Element>(target.size());
        for (Element element : target) {
            String value = element.getAttributeValue(attrName);
            if (attrValue.equals(value)) {
                result.add(element);
            }
        }
        return result;
    }

    private List<Element> findElements(final List<Element> target,
            final String elementName) {
        List<Element> result = new ArrayList<Element>(target.size());
        for (Element element : target) {
            String name = element.getName();
            if (elementName.equalsIgnoreCase(name)) {
                result.add(element);
            }
        }
        return result;
    }

    private List<Element> findElements(final List<Element> target,
            final String elementName, final String attrName,
            final String attrValue) {
        List<Element> result = new ArrayList<Element>(target.size());
        for (Element element : target) {
            String name = element.getName();
            String value = element.getAttributeValue(attrName);
            if (elementName.equalsIgnoreCase(name) && attrValue.equals(value)) {
                result.add(element);
            }
        }
        return result;
    }
}
