package jp.littleforest.util.http;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import jp.littleforest.util.AssertionUtil;
import jp.littleforest.util.encode.EncodingEnum;
import jp.littleforest.util.exception.HttpUtilException;
import jp.littleforest.util.io.FileUtil;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

/**
 * HTTPアクセスをサポートするためのユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class HttpUtil {
    private HttpUtil() {

    }

    /**
     * 指定した URL から HTML を取得し、ボディを文字列として返します。<br />
     * 
     * エンコーディングには、 {@link Encoding#ENCODING_JIS_AUTO_DETECT} が指定されます。<br />
     * リダイレクトに従います。<br />
     * 
     * @param url URL
     * @return レスポンスボディ。ステータスが OK 以外の場合は <code>null</code>。
     */
    public static String getHtmlBody(final String url) {
        HttpResponce res = getHtml(url);
        if (res != null && res.getStatus() == HttpResponce.SC_OK) {
            return res.getBody();
        } else {
            return null;
        }
    }

    /**
     * 指定した URL から HTML を取得し、ボディを文字列として返します。<br />
     * 
     * @param url URL
     * @param encoding エンコーディング
     * @param followRedirects リダイレクト指定
     * @return レスポンスボディ。ステータスが OK 以外の場合は <code>null</code>。
     */
    public static String getHtmlBody(final String url, final String encoding,
            final boolean followRedirects) {
        HttpResponce res = getHtml(url, encoding, followRedirects);
        if (res != null && res.getStatus() == HttpResponce.SC_OK) {
            return res.getBody();
        } else {
            return null;
        }
    }

    /**
     * 指定した URL から HTML を取得します。<br />
     * 
     * エンコーディングには、 {@link Encoding#ENCODING_JIS_AUTO_DETECT} が指定されます。<br />
     * リダイレクトに従います。<br />
     * 
     * @param url URL
     * @return {@link HttpResponce} オブジェクト
     * @throws HttpUtilException
     * @see {@link Encoding}
     */
    public static HttpResponce getHtml(final String url) {
        return getHtml(url, EncodingEnum.JIS_AUTO_DETECT.name, true);
    }

    /**
     * 指定した URL から HTML を取得します。<br />
     * 
     * @param url URL
     * @param encoding エンコーディング
     * @return レスポンスボディ(取得できなかった場合は <code>null</code>)
     * @throws HttpUtilException
     * @see {@link Encoding}
     */
    public static HttpResponce getHtml(final String url,
            final EncodingEnum encoding, final boolean followRedirects) {
        AssertionUtil.assertNotNull("encoding", encoding);
        return getHtml(url, encoding.name, followRedirects);
    }

    /**
     * 指定した URL から HTML を取得します。<br />
     * 
     * @param url URL
     * @param encoding エンコーディング
     * @return レスポンスボディ(取得できなかった場合は <code>null</code>)
     * @throws HttpUtilException
     * @see {@link Encoding}
     */
    public static HttpResponce getHtml(final String url, final String encoding,
            final boolean followRedirects) {
        HttpClient client = new HttpClient();
        HttpMethod method = new GetMethod(url);
        method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
                new DefaultHttpMethodRetryHandler(3, false));
        method.setFollowRedirects(followRedirects);

        HttpResponce responce = getHtml(client, method, encoding);
        method.releaseConnection();
        return responce;
    }

    /**
     * 指定された {@link HttpClient}、 {@link HttpMethod} を利用して、HTML を取得します。<br />
     * コネクションの解放は行いません。<br />
     * 
     * @param client {@link HttpClient} オブジェクト
     * @param method {@link HttpMethod} オブジェクト
     * @param encoding エンコーディング
     * @return {@link HttpResponce} オブジェクト
     */
    public static HttpResponce getHtml(final HttpClient client,
            final HttpMethod method, final EncodingEnum encoding) {
        AssertionUtil.assertNotNull("encoding", encoding);
        return getHtml(client, method, encoding.name);
    }

    /**
     * 指定された {@link HttpClient}、 {@link HttpMethod} を利用して、HTML を取得します。<br />
     * コネクションの解放は行いません。<br />
     * 
     * @param client {@link HttpClient} オブジェクト
     * @param method {@link HttpMethod} オブジェクト
     * @param encoding エンコーディング
     * @return {@link HttpResponce} オブジェクト
     */
    public static HttpResponce getHtml(final HttpClient client,
            final HttpMethod method, final String encoding) {
        try {
            HttpResponce responce;
            int statusCode = client.executeMethod(method);
            StringBuilder html = new StringBuilder(2048);
            if (statusCode == HttpStatus.SC_OK) {
                InputStream is = method.getResponseBodyAsStream();
                BufferedReader reader = new BufferedReader(
                        new InputStreamReader(is, encoding));

                String line;
                while ((line = reader.readLine()) != null) {
                    html.append(line);
                    html.append("\n");
                }
                responce = new HttpResponce(statusCode, html.toString());
            } else {
                responce = new HttpResponce(statusCode, null);
            }
            return responce;
        } catch (HttpException ex) {
            throw new HttpUtilException(ex);
        } catch (IOException ex) {
            throw new HttpUtilException(ex);
        }
    }

    /**
     * 指定された URL からリソースを取得してファイルに保存します。<br />
     * 
     * @param url URL
     * @param dir 保存先ディレクトリ
     * @param fileName 保存先ファイル名。<code>null</code> を指定した場合は元のファイル名と同じ。
     * @param override <code>false</code> を指定した場合、すでにファイルが存在すればダウンロードしません。
     * @param followRedirects <code>true</code> を指定した場合、リダイレクトに従う。
     * @return ダウンロードが成功した場合、<code>true</code>。
     */
    public static boolean getResource(final String url, final String dir,
            final String fileName, final boolean override,
            final boolean followRedirects) {
        HttpClient client = new HttpClient();
        HttpMethod method = new GetMethod(url);
        method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
                new DefaultHttpMethodRetryHandler(3, false));
        method.setFollowRedirects(followRedirects);

        boolean result = getResource(client, method, dir, fileName, override);

        method.releaseConnection();
        return result;
    }

    /**
     * 指定された URL からリソースを取得してファイルに保存します。<br />
     * 
     * @param client {@link HttpClient} オブジェクト
     * @param method {@link HttpMethod} オブジェクト
     * @param dir 保存先ディレクトリ
     * @param fileName 保存先ファイル名。<code>null</code> を指定した場合は元のファイル名と同じ。
     * @param override <code>false</code> を指定した場合、すでにファイルが存在すればダウンロードしません。
     * @return ダウンロードが成功した場合、<code>true</code>。
     */
    public static boolean getResource(final HttpClient client,
            final HttpMethod method, final String dir, final String fileName,
            final boolean override) {
        try {
            String url = method.getURI().getPath();
            String outFileName = dir + "/" + fileName;
            if (outFileName == null) {
                int pos = url.lastIndexOf("/");
                if (pos > -1) {
                    outFileName = dir + "/" + url.substring(pos + 1);
                }
            }

            if (!override && FileUtil.exists(outFileName)) {
                return false;
            }

            FileUtil.prepareDir(dir);

            int statusCode = client.executeMethod(method);
            if (statusCode == HttpStatus.SC_OK) {
                BufferedInputStream is = new BufferedInputStream(method
                        .getResponseBodyAsStream());
                BufferedOutputStream os = new BufferedOutputStream(
                        new FileOutputStream(outFileName));
                int len = -1;
                byte[] buf = new byte[8 * 1024];
                try {
                    while ((len = is.read(buf, 0, buf.length)) != -1) {
                        os.write(buf, 0, len);
                    }
                    os.flush();
                } finally {
                    if (is != null) {
                        is.close();
                    }
                    if (os != null) {
                        os.close();
                    }
                }
                return true;
            } else {
                return false;
            }
        } catch (HttpException ex) {
            throw new HttpUtilException(ex);
        } catch (IOException ex) {
            throw new HttpUtilException(ex);
        }
    }
}
