package jp.littleforest.util.image;

import java.util.ArrayList;
import java.util.List;

import jp.littleforest.util.image.impl.BiLinearScaler;
import jp.littleforest.util.image.impl.LanczosScaler;

import org.eclipse.swt.graphics.ImageData;

/**
 * イメージのスケーリングを行うためのクラスです。<br />
 * 
 * @author y-komori
 */
public class SWTImageScaler {
    /**
     * スケーリングアルゴリズムを表す列挙型です。<br />
     * 
     * @author y-komori
     */
    public enum ScalingAlgorithm {
        /** バイリニア法を使用します。 */
        BI_LINEAR(BiLinearScaler.class.getName()),

        /** ランツォシュ法を使用します */
        LANCZOS(LanczosScaler.class.getName());

        private String className;

        private ScalingAlgorithm(final String className) {
            this.className = className;
        }

        public String getClassName() {
            return className;
        }
    }

    private static final List<Scaler> scalers = new ArrayList<Scaler>();

    private static boolean initialized = false;

    private static boolean isInitialized() {
        return initialized;
    }

    private static synchronized void init() {
        scalers.add(new BiLinearScaler());
        scalers.add(new LanczosScaler());
    }

    /**
     * 指定したサイズに収まるようにイメージのスケーリングを行います。<br />
     * 
     * @param image
     *            処理対象イメージ
     * @param width
     *            スケーリング後の幅
     * @param heigt
     *            スケーリング後の高さ
     * @param algo
     *            アルゴリズム
     * @return スケーリングされたイメージ
     */
    public static ImageData scaleImageKeepingAspectRatio(final ImageData image,
            final int width, final int height, final ScalingAlgorithm algo) {
        if (image == null) {
            throw new IllegalArgumentException("image can't be null");
        }

        float xScale = (float) width / (float) image.width;
        float yScale = (float) height / (float) image.height;
        float scale = Math.min(xScale, yScale);
        return scaleImage(image, scale, scale, algo);
    }

    /**
     * サイズを指定したイメージのスケーリングを行います。<br />
     * 
     * @param image
     *            処理対象イメージ
     * @param width
     *            スケーリング後の幅
     * @param heigt
     *            スケーリング後の高さ
     * @param algo
     *            アルゴリズム
     * @return スケーリングされたイメージ
     */
    public static ImageData scaleImage(final ImageData image, final int width,
            final int height, final ScalingAlgorithm algo) {
        if (image == null) {
            throw new IllegalArgumentException("image can't be null");
        }

        float xScale = (float) width / (float) image.width;
        float yScale = (float) height / (float) image.height;
        return scaleImage(image, xScale, yScale, algo);
    }

    /**
     * イメージのスケーリングを行います。<br />
     * 
     * @param image
     *            処理対象イメージ
     * @param scale
     *            スケーリング係数
     * @param algo
     *            アルゴリズム
     * @return スケーリングされたイメージ
     */
    public static ImageData scaleImage(final ImageData image,
            final float scale, final ScalingAlgorithm algo) {
        return scaleImage(image, scale, scale, algo);
    }

    private static Scaler findScaler(final ScalingAlgorithm algo) {
        String className = algo.getClassName();
        for (Scaler scaler : scalers) {
            if (scaler.getClass().getName().equals(className)) {
                return scaler;
            }
        }
        return null;
    }

    private static ImageData scaleImage(final ImageData image,
            final float xScale, final float yScale, final ScalingAlgorithm algo) {
        if (!isInitialized()) {
            init();
        }

        Scaler scaler = findScaler(algo);
        return scaler.scaleImage(image, xScale, yScale);
    }
}
