package jp.littleforest.util.io;

import java.io.Closeable;
import java.io.DataOutput;
import java.io.FileOutputStream;
import java.io.Flushable;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;

/**
 * @author y-komori
 * 
 */
public class ByteBufferOutputStream implements DataOutput, Closeable, Flushable {
    private ByteBuffer buffer;

    private FileChannel channel;

    public ByteBufferOutputStream(FileOutputStream out, int capacity) {
        if (out == null || capacity < 8) {
            throw new IllegalArgumentException();
        }
        channel = out.getChannel();
        buffer = ByteBuffer.allocate(capacity);
    }

    public void order(ByteOrder order) {
        buffer.order(order);
    }

    public void write(int b) throws IOException {
        if (buffer.remaining() < 4) {
            writeBuffer();
        }
        buffer.putInt(b);
    }

    public void write(byte[] b) throws IOException {
        write(b, 0, b.length);
    }

    public void write(byte[] b, int off, int len) throws IOException {
        int pos = off;
        int remaind = off + len;
        while (remaind > 0) {
            if (buffer.remaining() >= remaind) {
                buffer.put(b, pos, remaind);
                remaind = remaind - b.length;
            } else {
                int capableSize = buffer.remaining();
                buffer.put(b, pos, capableSize);
                pos += capableSize;
                remaind -= capableSize;
                writeBuffer();
            }
        }
    }

    public void writeBoolean(boolean v) throws IOException {
        if (buffer.remaining() < 1) {
            writeBuffer();
        }
        buffer.put((byte) (v ? 1 : 0));
    }

    public void writeByte(int v) throws IOException {
        if (buffer.remaining() < 1) {
            writeBuffer();
        }
        buffer.put((byte) v);
    }

    public void writeBytes(String s) throws IOException {
        int len = s.length();
        for (int i = 0; i < len; i++) {
            if (buffer.remaining() < 1) {
                writeBuffer();
            }
            buffer.put((byte) s.charAt(i));
        }
    }

    public void writeChar(int v) throws IOException {
        if (buffer.remaining() < 2) {
            writeBuffer();
        }
        buffer.put((byte) ((v >>> 8) & 0xFF));
        buffer.put((byte) ((v >>> 0) & 0xFF));
    }

    public void writeChars(String s) throws IOException {
        int len = s.length();
        for (int i = 0; i < len; i++) {
            if (buffer.remaining() < 2) {
                writeBuffer();
            }
            int v = s.charAt(i);
            buffer.put((byte) ((v >>> 8) & 0xFF));
            buffer.put((byte) ((v >>> 0) & 0xFF));
        }
    }

    public void writeDouble(double v) throws IOException {
        if (buffer.remaining() < 8) {
            writeBuffer();
        }
        buffer.putDouble(v);
    }

    public void writeFloat(float v) throws IOException {
        if (buffer.remaining() < 4) {
            writeBuffer();
        }
        buffer.putFloat(4);
    }

    public void writeInt(int v) throws IOException {
        if (buffer.remaining() < 4) {
            writeBuffer();
        }
        buffer.putInt(v);
    }

    public void writeLong(long v) throws IOException {
        if (buffer.remaining() < 8) {
            writeBuffer();
        }
        buffer.putLong(v);
    }

    public void writeShort(int v) throws IOException {
        if (buffer.remaining() < 2) {
            writeBuffer();
        }
        buffer.putShort((short) v);
    }

    public void writeUTF(String str) throws IOException {
        throw new UnsupportedOperationException();
    }

    public void close() throws IOException {
        writeBuffer();
        channel.close();
    }

    public void flush() throws IOException {
        writeBuffer();
    }

    private void writeBuffer() throws IOException {
        buffer.limit(buffer.position());
        buffer.rewind();
        channel.write(buffer.slice());
        buffer.clear();
    }
}
