package jp.littleforest.util.io;

/**
 * ディレクトリのステータスを保持するクラスです。<br />
 * 
 * @author y-komori
 */
public class DirStatus {
    private final int dirNum;

    private final int fileNum;

    private final int childrenFileNum;

    private final long fileSize;

    private final long dirSize;

    /**
     * {@link DirStatus} を構築します。<br />
     * 
     * @param dirNum
     *        直下のディレクトリの数
     * @param fileNum
     *        直下のファイルの数
     * @param childrenFileNum
     *        配下のディレクトリが含むファイルの総数
     * @param fileSize
     *        直下のファイルの総バイト数
     * @param dirSize
     *        配下のディレクトリが含むファイルの総バイト数
     */
    public DirStatus(final int dirNum, final int fileNum,
            final int childrenFileNum, final long fileSize, final long dirSize) {
        this.dirNum = dirNum;
        this.fileNum = fileNum;
        this.childrenFileNum = childrenFileNum;
        this.fileSize = fileSize;
        this.dirSize = dirSize;
    }

    /**
     * {@link DirStatus} を構築します。<br />
     */
    public DirStatus() {
        this.dirNum = -1;
        this.fileNum = -1;
        this.childrenFileNum = -1;
        this.fileSize = -1;
        this.dirSize = -1;
    }

    /**
     * 直下のディレクトリの数を返します。<br />
     * 
     * @return 直下のディレクトリの数
     */
    public int getDirNum() {
        return dirNum;
    }

    /**
     * 直下のファイルの数を返します。<br />
     * 
     * @return 直下のファイルの数
     */
    public int getFileNum() {
        return fileNum;
    }

    /**
     * 配下のディレクトリが含むファイルの総数を返します。<br />
     * 
     * @return 配下のディレクトリが含むファイルの総数
     */
    public int getChildrenFileNum() {
        return childrenFileNum;
    }

    /**
     * 配下も含めたファイルの総数を返します。<br />
     * ディレクトリの数は含めません。
     * 
     * @return 配下も含めたファイルの総数
     */
    public int getTotalFileNum() {
        return fileNum + childrenFileNum;
    }

    /**
     * 直下のファイルの総バイト数を返します。<br />
     * 
     * @return 配下のファイルの総バイト数
     */
    public long getFileSize() {
        return fileSize;
    }

    /**
     * 配下のディレクトリが含むファイルの総バイト数を返します。<br />
     * 
     * @return 配下のディレクトリが含むファイルの総バイト数
     */
    public long getDirSize() {
        return dirSize;
    }

    /**
     * このディレクトリが含むファイルの総バイト数を返します。<br />
     * 
     * @return このディレクトリが含むファイルの総バイト数
     */
    public long getTotalSize() {
        return fileSize + dirSize;
    }

}
