/**
 * Copyright 2009 Yusuke Komori. All rights reserved.
 */
package jp.littleforest.util.io;

import static jp.littleforest.util.encode.EncodingEnum.*;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

/**
 * エンコーディングを指定することのできる {@link FileReader} です。<br />
 * 
 * @author y-komori
 */
public class EncodeFileReader extends InputStreamReader {

    /**
     * 読み込み元のファイルの名前を指定して、新規 {@link EncodeFileReader} を作成します。<br />
     * エンコーディングには <code>x-JISAutoDetect</code> が使用されます。
     * 
     * @param fileName 読み込み元のファイルの名前
     * @throws FileNotFoundException ファイルが存在しないか、普通のファイルでは
     *             なくディレクトリであるか、または何らかの理由で開くことができない場合
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final String fileName)
            throws FileNotFoundException, UnsupportedEncodingException {
        super(new FileInputStream(fileName), X_JIS_AUTO_DETECT.name);
    }

    /**
     * 読み込み元のファイルの名前を指定して、新規 <tt>FileReader</tt> を作成します。
     * 
     * @param fileName 読み込み元のファイルの名前
     * @param charsetName サポートする {@link java.nio.charset.Charset
     *            <code>charset</code>} の名前
     * @throws FileNotFoundException ファイルが存在しないか、普通のファイルでは
     *             なくディレクトリであるか、または何らかの理由で開くことができない場合
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final String fileName, final String charsetName)
            throws FileNotFoundException, UnsupportedEncodingException {
        super(new FileInputStream(fileName), charsetName);
    }

    /**
     * 読み込み元の {@link File} を指定して、新規 {@link EncodeFileReader} を作成します。<br />
     * エンコーディングには <code>x-JISAutoDetect</code> が使用されます。
     * 
     * @param file 読み込み元の <tt>File</tt>
     * @throws FileNotFoundException ファイルが存在しないか、普通のファイルでは
     *             なくディレクトリであるか、または何らかの理由で開くことができない場合
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final File file) throws FileNotFoundException,
            UnsupportedEncodingException {
        super(new FileInputStream(file), X_JIS_AUTO_DETECT.name);
    }

    /**
     * 読み込み元の {@link File} を指定して、新規 {@link EncodeFileReader} を作成します。<br />
     * 
     * @param file 読み込み元の <tt>File</tt>
     * @param charsetName サポートする {@link java.nio.charset.Charset
     *            <code>charset</code>} の名前
     * @throws FileNotFoundException ファイルが存在しないか、普通のファイルでは
     *             なくディレクトリであるか、または何らかの理由で開くことができない場合
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final File file, final String charsetName)
            throws FileNotFoundException, UnsupportedEncodingException {
        super(new FileInputStream(file), charsetName);
    }

    /**
     * 読み込み元の {@link FileDescriptor} を指定して、新規 {@link FileReader} を作成します。<br />
     * エンコーディングには <code>x-JISAutoDetect</code> が使用されます。
     * 
     * @param fd 読み込み元の FileDescriptor
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final FileDescriptor fd)
            throws UnsupportedEncodingException {
        super(new FileInputStream(fd), X_JIS_AUTO_DETECT.name);
    }

    /**
     * 読み込み元の {@link FileDescriptor} を指定して、新規 {@link FileReader} を作成します。<br />
     * 
     * @param fd 読み込み元の FileDescriptor
     * @param charsetName サポートする {@link java.nio.charset.Charset
     *            <code>charset</code>} の名前
     * @throws UnsupportedEncodingException 指定された文字セットがサポートされていない場合
     */
    public EncodeFileReader(final FileDescriptor fd, final String charsetName)
            throws UnsupportedEncodingException {
        super(new FileInputStream(fd), charsetName);
    }
}
