/**
 * Copyright 2009 Yusuke Komori. All rights reserved.
 */
package jp.littleforest.util.io;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;

/**
 * エンコーディングを指定してファイル出力を行う {@link FileWriter} です。<br />
 * 
 * @author y-komori
 */
public class EncodeFileWriter extends OutputStreamWriter {

    /**
     * ファイル名を指定して FileWriter オブジェクトを構築します。
     * 
     * @param fileName システムに依存するファイル名の文字列
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     */
    public EncodeFileWriter(final String fileName) throws IOException {
        super(new FileOutputStream(fileName));
    }

    /**
     * 書き込まれたデータを追加するかどうかを示す boolean とファイル名を 指定して、EncodeFileWriter オブジェクトを構築します。
     * 
     * @param fileName システムに依存するファイル名の文字列
     * @param append boolean が <code>true</code> の場合、データはファイルの 先頭ではなく最後に書き込まれる
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     */
    public EncodeFileWriter(final String fileName, final boolean append)
            throws IOException {
        super(new FileOutputStream(fileName, append));
    }

    /**
     * 書き込まれたデータを追加するかどうかを示す boolean とファイル名を 指定して、EncodeFileWriter オブジェクトを構築します。
     * 
     * @param fileName システムに依存するファイル名の文字列
     * @param append boolean が <code>true</code> の場合、データはファイルの 先頭ではなく最後に書き込まれる
     * @param charsetName サポートする {@link java.nio.charset.Charset
     *            <code>charset</code>} の名前
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     */
    public EncodeFileWriter(final String fileName, final boolean append,
            final String charsetName) throws IOException {
        super(new FileOutputStream(fileName, append), charsetName);
    }

    /**
     * File オブジェクトを指定して EncodeFileWriter オブジェクトを構築します。
     * 
     * @param file 書き込み先 File オブジェクト
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     */
    public EncodeFileWriter(final File file) throws IOException {
        super(new FileOutputStream(file));
    }

    /**
     * File オブジェクトを指定して EncodeFileWriter オブジェクトを構築します。 2 番目の引数が
     * <code>true</code> の場合、バイトはファイルの先頭では なく最後に書き込まれます。
     * 
     * @param file 書き込み先 File オブジェクト
     * @param append <code>true</code> の場合、バイトはファイルの先頭では なく最後に書き込まれる
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     * @since 1.4
     */
    public EncodeFileWriter(final File file, final boolean append)
            throws IOException {
        super(new FileOutputStream(file, append));
    }

    /**
     * File オブジェクトを指定して EncodeFileWriter オブジェクトを構築します。 2 番目の引数が
     * <code>true</code> の場合、バイトはファイルの先頭では なく最後に書き込まれます。
     * 
     * @param file 書き込み先 File オブジェクト
     * @param append <code>true</code> の場合、バイトはファイルの先頭では なく最後に書き込まれる
     * @param charsetName サポートする {@link java.nio.charset.Charset
     *            <code>charset</code>} の名前
     * @throws IOException ファイルは存在するが、普通のファイルではなくディレク
     *             トリである場合、ファイルは存在せず作成もできない場合、または 何らかの理由で開くことができない場合
     * @since 1.4
     */
    public EncodeFileWriter(final File file, final boolean append,
            final String charsetName) throws IOException {
        super(new FileOutputStream(file, append), charsetName);
    }

    /**
     * ファイル記述子に関連した EncodeFileWriter オブジェクトを構築します。
     * 
     * @param fd 書き込み先 FileDescriptor オブジェクト
     */
    public EncodeFileWriter(final FileDescriptor fd) {
        super(new FileOutputStream(fd));
    }

}
