package jp.littleforest.util.io;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Random;

import jp.co.ntt.isl.camellia.Camellia;
import jp.littleforest.util.AssertionUtil;

/**
 * データの暗号化/復号化を行うためのユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class EncryptUtil {
    private EncryptUtil() {

    }

    private static final Random random = new Random(System.currentTimeMillis());

    private static final ByteOrder ENDIAN = ByteOrder.BIG_ENDIAN;

    private static final int INT_SIZE = 4;

    /**
     * 指定されたバイト配列を暗号化します。<br />
     * 
     * @param target
     *        暗号化するバイト配列
     * @param key
     *        暗号化に使用するキー
     * @return 暗号化後のバイト配列
     */
    public static byte[] encrypt(final byte[] target, final byte[] key) {
        AssertionUtil.assertNotNull("target", target);
        AssertionUtil.assertNotNull("key", key);

        checkKeyLength(key);

        // Camellia の初期化
        Camellia camellia = new Camellia();
        camellia.init(true, key);
        int blockSize = camellia.getBlockSize();

        // 対象のバイト配列をブロック長の倍数に揃える
        int targetLength = calcResultLength(target.length, blockSize);
        byte[] paddedTarget = new byte[targetLength];
        System.arraycopy(target, 0, paddedTarget, 0, target.length);

        ByteBuffer out = ByteBuffer.allocate(targetLength + INT_SIZE * 2);
        byte[] outArray = out.array();
        out.order(ENDIAN);

        int padding = random.nextInt();

        // 最初の 4 バイト長隠蔽用のランダム値を格納
        out.putInt(padding);

        // 次の4バイトに隠蔽したオリジナルバイト長を格納
        out.putInt(target.length + padding);

        for (int inPos = 0, outPos = INT_SIZE * 2; inPos < targetLength; inPos += blockSize, outPos += blockSize) {
            camellia.processBlock(paddedTarget, inPos, outArray, outPos);
        }

        return outArray;
    }

    /**
     * 指定された文字列を暗号化します。<br />
     * 
     * @param target
     *        暗号化する文字列
     * @param key
     *        暗号化に使用するキー
     * @return 暗号化後のバイト配列
     */
    public static byte[] encrypt(final String target, final String key) {
        return encrypt(target.getBytes(), key.getBytes());
    }

    /**
     * 指定されたバイト配列を復号します。<br />
     * 
     * @param target
     *        復号するバイト配列
     * @return 復号後のバイト配列
     */
    public static byte[] decrypt(final byte[] target, final byte[] key) {
        AssertionUtil.assertNotNull("target", target);
        AssertionUtil.assertNotNull("key", key);

        checkKeyLength(key);

        // Camellia の初期化
        Camellia camellia = new Camellia();
        camellia.init(false, key);
        int blockSize = camellia.getBlockSize();

        // オリジナルのバイト長を取得する
        ByteBuffer in = ByteBuffer.wrap(target);
        in.order(ENDIAN);
        int padding = in.getInt();
        int orgLength = in.getInt() - padding;

        // 出力バッファの用意
        ByteBuffer out = ByteBuffer.allocate(orgLength);
        byte[] tmp = new byte[blockSize];

        for (int inPos = INT_SIZE * 2, remain = orgLength; inPos < target.length; inPos += blockSize, remain -= blockSize) {
            camellia.processBlock(target, inPos, tmp, 0);

            if (remain >= blockSize) {
                out.put(tmp);
            } else {
                out.put(tmp, 0, remain);
            }
        }

        return out.array();
    }

    /**
     * 暗号化されたバイト配列を、指定された文字列をキーとして復号します。<br />
     * 
     * @param target
     *        復号するバイト配列
     * @param key
     *        暗号化キー
     * @return 複合後の文字列
     */
    public static String decrypt(final byte[] target, final String key) {
        byte[] result = decrypt(target, key.getBytes());
        return new String(result);
    }

    /**
     * 128 bit のランダムなキーを生成します。<br />
     * 
     * @return キーのバイト配列
     */
    public static byte[] createRandomKey16() {
        return createRandomKey(16);
    }

    /**
     * 192 bit のランダムなキーを生成します。<br />
     * 
     * @return キーのバイト配列
     */
    public static byte[] createRandomKey24() {
        return createRandomKey(24);
    }

    /**
     * 256 bit のランダムなキーを生成します。<br />
     * 
     * @return キーのバイト配列
     */
    public static byte[] createRandomKey32() {
        return createRandomKey(32);
    }

    private static byte[] createRandomKey(final int size) {
        byte[] key = new byte[size];
        random.nextBytes(key);
        return key;
    }

    public static void checkKeyLength(final byte[] key) {
        if (key.length != 16 && key.length != 24 && key.length != 32) {
            throw new IllegalArgumentException(
                    "Key length must be 16, 24, or 32 bytes.");
        }
    }

    private static int calcResultLength(final int targetLength,
            final int blockSize) {
        int resultLength = targetLength;
        if ((targetLength % blockSize) == 0) {
            // 対象配列がブロックサイズで割りきれる場合
            return resultLength;
        } else {
            // 対象配列がブロックサイズで割りきれない場合
            resultLength = ((targetLength / blockSize) + 1) * blockSize;
        }
        return resultLength;
    }
}
