package jp.littleforest.util.io;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import jp.littleforest.util.AssertionUtil;

/**
 * ストアされたキーによる暗号化/復号化を行うためのクラスです。<br />
 * 
 * @author y-komori
 */
public class Encryptor {
    private final Random random = new Random(System.currentTimeMillis());

    private final List<byte[]> keys = new ArrayList<byte[]>();

    /**
     * 暗号化/復号化用のキーを追加します。<br />
     * キーの長さは 16 バイト、24 バイト、32 バイトのいずれかでなくては いけません。<br />
     * キーは最大で256個まで追加することができます。<br />
     * 
     * @param key
     *        キー
     */
    public synchronized void storeKey(final byte[] key) {
        AssertionUtil.assertNotNull("key", key);

        EncryptUtil.checkKeyLength(key);

        if (keys.size() < 0xff) {
            keys.add(key);
        } else {
            throw new IllegalArgumentException("Key store is full.");
        }
    }

    /**
     * ストアされたキーをすべて削除します。<br />
     */
    public synchronized void clearStoredKey() {
        keys.clear();
    }

    /**
     * 指定されたバイト配列を暗号化します。<br />
     * 暗号化キーは設定されたキーからランダムで選択され、キー番号が暗号化後バイト配列の第一バイト目に出力されます。<br />
     * 
     * @param target
     *        暗号化するバイト配列
     * @return 暗号化後のバイト配列
     */
    public byte[] encryptWithStoredKey(final byte[] target) {
        int index = selectKey();
        byte[] key = getKey(index);

        byte[] crypto = EncryptUtil.encrypt(target, key);
        byte[] result = new byte[crypto.length + 1];
        result[0] = (byte) (0x000000ff & index);
        System.arraycopy(crypto, 0, result, 1, crypto.length);

        return result;
    }

    /**
     * 指定された文字列を {@link #encryptWithStoredKey(byte[]) encryptWithStoredKey}
     * メソッドで暗号化します。<br />
     * 
     * @param target
     *        暗号化する文字列
     * @return 暗号化後のバイト配列
     */
    public byte[] encryptWithStoredKey(final String target) {
        return encryptWithStoredKey(target.getBytes());
    }

    /**
     * {@link #encryptWithStoredKey(byte[]) encryptWithStoredKey}
     * メソッドで暗号化されたバイト配列を復号します。<br />
     * 
     * @param target
     *        復号するバイト配列
     * @return 暗号化後のバイト配列
     */
    public byte[] decryptWithStoredKey(final byte[] target) {
        ByteBuffer buf = ByteBuffer.wrap(target);
        int keyIndex = buf.get();
        if (keyIndex < 0) {
            keyIndex = 0x100 + keyIndex;
        }
        byte[] key = getKey(keyIndex);
        byte[] encrypted = new byte[target.length - 1];
        buf.get(encrypted);
        return EncryptUtil.decrypt(encrypted, key);
    }

    private synchronized int selectKey() {
        if (keys.size() == 0) {
            throw new IllegalStateException("Key store is empty.");
        }
        return random.nextInt(keys.size());
    }

    private byte[] getKey(final int index) {
        if (index > keys.size()) {
            throw new IllegalArgumentException(
                    "Specified key index is not stored. (index:" + index + ")");
        }
        return keys.get(index);
    }

}
