package jp.littleforest.util.io;

import static jp.littleforest.util.encode.EncodingEnum.*;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

/**
 * ファイルやストリームに関するユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class FileUtil {
    private static final Map<File, String> volumeNameMap = new TreeMap<File, String>();

    public static void init() {
        init(File.listRoots());
    }

    public static void init(final File[] roots) {
        for (File drive : roots) {
            String path = drive.getPath();
            String label = "";
            try {
                ProcessBuilder builder = new ProcessBuilder("cmd", "/c", "dir",
                        path, "/A:-D-R-H-A-S");
                builder.redirectErrorStream(true);
                Process process = builder.start();
                BufferedReader in = new BufferedReader(new InputStreamReader(
                        process.getInputStream(), MS932.name));
                String line = in.readLine();
                if (line != null) {
                    int startPos = line.indexOf("ラベルは ");
                    int endPos = line.indexOf(" です");
                    if ((startPos > 0) && (endPos > 0)) {
                        label = line.substring(startPos + 5, endPos);
                    }
                }
            } catch (IOException ex) {
            }
            volumeNameMap.put(drive, label);
        }
    }

    /**
     * ドライブのボリュームラベルを取得します。<br />
     * 
     * @param root ボリュームラベルを取得するドライブのルートに対応する {@link File} オブジェクト
     * @return ボリュームラベル
     */
    public static String getVolumeLabel(final File root) {
        return volumeNameMap.get(root);
    }

    /**
     * 現在利用可能なドライブの一覧を返します。<br />
     * 
     * @return 利用可能なドライブに対応する {@link File} オブジェクトの配列
     */
    public static File[] getAvailableDrives() {
        ArrayList<File> drives = new ArrayList<File>();
        for (Entry<File, String> entry : volumeNameMap.entrySet()) {
            if (!entry.getValue().equals("")) {
                drives.add(entry.getKey());
            }
        }
        File[] result = new File[drives.size()];
        return drives.toArray(result);
    }

    /**
     * 指定されたパスの示すファイルが存在するかどうかチェックします。<br />
     * 
     * @param path ファイルのパス
     * @return 存在する場合、<code>true</code>
     */
    public static boolean exists(final String path) {
        if (path != null) {
            return (new File(path)).exists();
        } else {
            return false;
        }
    }

    /**
     * 指定されたパスの示すディレクトリが存在しない場合、新たに作成します。<br />
     * 
     * @param path ディレクトリのパス
     */
    public static void prepareDir(final String path) {
        File dir = new File(path);
        if (!dir.exists()) {
            dir.mkdirs();
        }
    }

    /**
     * 指定されたパスの拡張子を返します。<br />
     * 
     * @param path パス
     * @return 拡張子。拡張子がない場合は空文字列。
     */
    public static String getExtension(final String path) {
        if (path == null) {
            throw new IllegalArgumentException("path can't be null");
        }
        int pos = path.lastIndexOf(".");
        if (pos > -1) {
            return path.substring(pos + 1);
        } else {
            return "";
        }
    }

    /**
     * 指定されたパスの拡張子を除いた部分を返します。<br />
     * 
     * @param path パス
     * @return ベースネーム
     */
    public static String getBasename(final String path) {
        if (path == null) {
            throw new IllegalArgumentException("path can't be null");
        }
        int pos = path.lastIndexOf(".");
        if (pos > -1) {
            return path.substring(0, pos);
        } else {
            return path;
        }
    }

    /**
     * パスで指定されたファイルの内容を指定されたバッファへ読み込みます。<br />
     * 
     * @param path パス
     * @param buff バッファ
     * @return 読み込んだ結果
     * @throws IOException 入出力エラーが発生した場合
     */
    public static byte[] readFile(final String path, final byte[] buff)
            throws IOException {
        if (path == null) {
            throw new IllegalArgumentException("path can't be null");
        }
        if (buff == null) {
            throw new IllegalArgumentException("buff can't be null");
        }

        File file = new File(path);
        if (buff.length < file.length()) {
            throw new IllegalArgumentException(
                    "buff's capacity is not much enough");
        }

        int bufSize = 4096; // バッファサイズ
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(
                file), bufSize);
        int length = (int) file.length();

        try {
            int off = 0; // 配列中の読込位置
            int readByte = 0; // 読み込んだバイト数
            int remain = length; // 残りバイト数
            int readSize = remain < bufSize ? remain : bufSize; // 次に読み込むバイト数
            while ((readByte = bis.read(buff, off, readSize)) != -1) {
                off += readByte;
                remain -= readByte;
                if (readSize > remain) {
                    readSize = remain;
                }
                if (remain <= 0) {
                    break;
                }
            }
        } catch (IOException ex) {
            closeStream(bis);
            throw ex;
        } finally {
            closeStream(bis);
        }

        return buff;
    }

    /**
     * 指定されたパスが示すファイルの内容を読み込み、文字列として返します。<br />
     * エンコーディングには <code>x-JISAutoDetect</code> を使用します。
     * 
     * @param path パス
     * @return ファイルの内容
     * @throws IOException 入出力例外が発生した場合
     */
    public static String readFileToString(final String path) throws IOException {
        return readFileToString(path, null);
    }

    /**
     * 指定されたパスが示すファイルの内容を読み込み、文字列として返します。<br />
     * 
     * @param path パス
     * @param charsetName エンコーディング。<code>null</code> の場合は
     *            <code>x-JISAutoDetect</code> を使用する。
     * @return ファイルの内容
     * @throws IOException 入出力例外が発生した場合
     */
    public static String readFileToString(final String path,
            final String charsetName) throws IOException {
        BufferedReader reader = null;
        String encoding = charsetName;
        if (encoding == null) {
            encoding = X_JIS_AUTO_DETECT.name;
        }

        StringBuffer buf = new StringBuffer(4096);
        try {
            reader = new BufferedReader(new EncodeFileReader(path, encoding));

            String line;
            while ((line = reader.readLine()) != null) {
                buf.append(line);
                buf.append("\n");
            }
        } catch (IOException ex) {
            if (reader != null) {
                closeStream(reader);
            }
            throw ex;
        } finally {
            if (reader != null) {
                closeStream(reader);
            }
        }

        return buf.toString();
    }

    /**
     * パスで指定されたファイルの内容をバイト配列へ読み込みます。<br />
     * 
     * @param path パス
     * @return 読み込んだ結果
     * @throws IOException 入出力エラーが発生した場合
     */
    public static byte[] readFile(final String path) throws IOException {
        File file = new File(path);
        byte[] buff = new byte[(int) file.length()];
        return readFile(path, buff);
    }

    /**
     * ストリームをクローズします。<br />
     * 
     * ストリームが <code>null</code> の場合は何も行いません。<br />
     * また、クローズ処理で例外が発生した場合も、何も行いません。
     * 
     * @param stream クローズ対象ストリーム
     */
    public static final void closeStream(final Closeable stream) {
        if (stream != null) {
            try {
                stream.close();
            } catch (IOException ex) {
                // Do nothing.
            }
        }
    }

    /**
     * 指定されたディレクトリに関するファイル数とファイルサイズに関する情報を取得します。<br />
     * 
     * @param dir ディレクトリ
     * @return {@link DirStatus} オブジェクト
     */
    public static DirStatus getDirStatus(final File dir) {
        int fileNum = 0;
        int dirNum = 0;
        int childrenFileNum = 0;
        long fileSize = 0;
        long dirSize = 0;

        if (dir.isDirectory()) {
            for (File file : dir.listFiles()) {
                if (file.isFile()) {
                    fileNum++;
                    fileSize += file.length();
                } else if (file.isDirectory()) {
                    dirNum++;
                    DirStatus dirStatus = getDirStatus(file);
                    childrenFileNum += dirStatus.getTotalFileNum();
                    dirSize += dirStatus.getTotalSize();
                }
            }
            return new DirStatus(dirNum, fileNum, childrenFileNum, fileSize,
                    dirSize);
        } else {
            throw new IllegalArgumentException("dir can't represents files.");
        }
    }
}
