/**
 * Copyright 2009 Yusuke Komori.
 */
package jp.littleforest.util.io;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import jp.littleforest.util.AssertionUtil;
import jp.littleforest.util.encode.EncodingEnum;
import jp.littleforest.util.exception.IORuntimeException;

/**
 * ストリームを扱うためのユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class StreamUtil {
    private StreamUtil() {

    }

    /**
     * {@link InputStream} から {@link InputStreamReader} を生成します。<br />
     * エンコーディングには {@link EncodingEnum#X_JIS_AUTO_DETECT} が使用されます。
     * 
     * @param in 入力ストリーム
     * @return エンコーディング
     */
    public static InputStreamReader createInputStreamReader(final InputStream in) {
        return createInputStreamReader(in, EncodingEnum.X_JIS_AUTO_DETECT);
    }

    /**
     * {@link InputStream} からエンコーディングを指定して {@link InputStreamReader} を生成します。<br />
     * 
     * @param in 入力ストリーム
     * @param encoding エンコーディング
     * @return {@link InputStreamReader} オブジェクト
     */
    public static InputStreamReader createInputStreamReader(
            final InputStream in, final EncodingEnum encoding) {
        AssertionUtil.assertNotNull("in", in);
        AssertionUtil.assertNotNull("encoding", encoding);

        InputStreamReader reader = null;
        try {
            reader = new InputStreamReader(in, encoding.name);
        } catch (UnsupportedEncodingException ex) {
            // Do nothing.
        }
        return reader;
    }

    /**
     * ストリームをクローズします。<br />
     * 
     * @param closeable ストリーム
     * @throws IORuntimeException クローズに失敗した場合
     */
    public static void closeStream(final Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            } catch (IOException ex) {
                throw new IORuntimeException(ex);
            }
        }
    }
}
