package jp.littleforest.util.resource;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jp.littleforest.util.resource.impl.FileResourceTraverser;

/**
 * {@link ResourceTraverser} のためのファクトリクラスです。<br />
 * 
 * @author y-komori
 */
public class ResourceTraverserFactory {
    private static final List<ResourceTraverser> traversers = new ArrayList<ResourceTraverser>();

    static {
        addResourceTraverser(new FileResourceTraverser());
    }

    private ResourceTraverserFactory() {

    }

    /**
     * {@link ResourceTraverser} を登録します。<br /> 同じプロトコルの
     * {@link ResourceTraverser} が既に登録されている場合、上書きします。<br />
     * 
     * @param traverser
     *            {@link ResourceTraverser}
     */
    public static void addResourceTraverser(final ResourceTraverser traverser) {
        ResourceTraverser known = getResourceTraverser(traverser.getProtocol());
        if (known != null) {
            traversers.remove(known);
        }
        traversers.add(traverser);
    }

    /**
     * 指定されたプロトコルに対応した {@link ResourceTraverser} を取得します。<br />
     * 
     * @param protocol
     *            プロトコル
     * @return {@link ResourceTraverser} オブジェクト。プロトコルに対応するものが存在しない場合は
     *         <code>null</code>。
     */
    public static ResourceTraverser getResourceTraverser(final String protocol) {
        if (protocol == null) {
            return null;
        }
        int size = traversers.size();
        for (int i = 0; i < size; i++) {
            ResourceTraverser traverser = traversers.get(i);
            if (protocol.equals(traverser.getProtocol())) {
                return traverser;
            }
        }
        return null;
    }

    /**
     * 指定された URL のプロトコルに対応した {@link ResourceTraverser} を取得します。<br />
     * 
     * @param url
     *            URL
     * @return {@link ResourceTraverser} オブジェクト。プロトコルに対応するものが存在しない場合は
     *         <code>null</code>。
     */
    public static ResourceTraverser getResourceTraverser(final URL url) {
        if (url == null) {
            return null;
        } else {
            return getResourceTraverser(url.getProtocol());
        }
    }
}
