/**
 * Copyright 2009 Yusuke Komori.
 */
package jp.littleforest.util.resource;

import java.io.File;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;

import jp.littleforest.util.AssertionUtil;
import jp.littleforest.util.exception.ResourceNotFoundException;

/**
 * リソースを扱うためのユーティリティクラスです。<br />
 * 
 * @author y-komori
 */
public class ResourceUtil {
    private ResourceUtil() {

    }

    /**
     * 指定されたクラスのパッケージを表すリソースパスを返します。<br />
     * 
     * @param clazz パッケージのリソースパスを取得するクラス
     * @return リソースパス
     */
    public static String getPackagePath(final Class<?> clazz) {
        if (clazz != null) {
            return "/" + clazz.getPackage().getName().replace('.', '/');
        } else {
            return "";
        }
    }

    /**
     * 指定されたクラスと同じパッケージにあるリソースの絶対パスを返します。<br />
     * 
     * @param clazz クラス
     * @param name リソース名
     * @return 絶対パス
     */
    public static String getAbsolutePath(final Class<?> clazz, final String name) {
        String prefix = getPackagePath(clazz);
        if (!"".equals(prefix)) {
            return prefix + "/" + name;
        } else {
            return name;
        }
    }

    /**
     * 指定されたクラスと同じパッケージにあるリソースを {@link File} オブジェクト として取得します。<br />
     * 
     * @param clazz リソースと同じパッケージのクラスオブジェクト
     * @param name リソース名
     * @return {@link File} オブジェクト
     */
    public static File getResourceAsFile(final Class<?> clazz, final String name) {
        String absolutePath = getAbsolutePath(clazz, name);
        try {
            URL url = clazz.getResource(absolutePath);
            if (url != null) {
                return new File(url.toURI());
            } else {
                return null;
            }
        } catch (URISyntaxException ex) {
            return null;
        }
    }

    /**
     * 指定されたクラスと同じパッケージにあるリソースを入力ストリームとして取得します。<br />
     * 
     * @param clazz リソースと同じパッケージのクラスオブジェクト
     * @param name リソース名
     * @throws ResourceNotFoundException リソースが見つからなかった場合
     * @return 入力ストリーム
     */
    public static InputStream getResourceAsStream(final Class<?> clazz,
            final String name) {
        String path = getAbsolutePath(clazz, name);
        InputStream in = clazz.getResourceAsStream(name);
        if (in != null) {
            return in;
        } else {
            throw new ResourceNotFoundException(path);
        }
    }

    /**
     * 指定された名前のルートパッケージに存在するリソースを入力ストリームとして取得します。<br />
     * クラスローダはコンテクストクラスローダを使用します。
     * 
     * @param name リソース名
     * @throws ResourceNotFoundException リソースが見つからなかった場合
     * @return 入力ストリーム
     */
    public static InputStream getResourceAsStream(final String name) {
        String path = getAbsolutePath(null, name);
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        InputStream in = loader.getResourceAsStream(name);
        if (in != null) {
            return in;
        } else {
            throw new ResourceNotFoundException(path);
        }
    }

    /**
     * 指定されたクラスと同じパッケージにあるリソースを入力ストリームとして取得します。<br />
     * リソース名は以下の形式として扱います。<br />
     * <br />
     * 
     * <i>(clazzで指定されたクラスのクラス名)(suffix)</i>[.<i>(ext)</i>]
     * 
     * <br />
     * <dl>
     * <dt>【例1】 clazz = FooClass, suffix = "_01", ext = "xml" の場合</dt>
     * <dd>→ FooClass_01.xml</dd>
     * <dt>【例2】 clazz = FooClass, suffix = null, ext = "xml" の場合</dt>
     * <dd>→ FooClass.xml</dd>
     * <dt>【例3】 clazz = FooClass, suffix = "_01", ext = null の場合</dt>
     * <dd>→ FooClass_01</dd>
     * </dl>
     * 
     * @param clazz リソースと同じパッケージのクラスオブジェクト
     * @param suffix サフィックス。<code>null</code> の場合は何もつけない
     * @param ext 拡張子。<code>null</code> の場合は何もつけない
     * @throws ResourceNotFoundException リソースが見つからなかった場合
     * @return 入力ストリーム
     */
    public static InputStream getResourceAsStream(final Class<?> clazz,
            final String suffix, final String ext) {
        AssertionUtil.assertNotNull("clazz", clazz);
        StringBuilder buf = new StringBuilder(32);
        buf.append(clazz.getSimpleName());
        if (suffix != null) {
            buf.append(suffix);
        }
        if (ext != null) {
            buf.append(".");
            buf.append(ext);
        }
        return getResourceAsStream(clazz, buf.toString());
    }
}
