package jp.littleforest.util.resource.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import jp.littleforest.util.exception.IORuntimeException;
import jp.littleforest.util.resource.ResourceFilter;
import jp.littleforest.util.resource.ResourceHandler;
import jp.littleforest.util.resource.ResourceTraverser;

/**
 * ファイルシステムをたどるための {@link ResourceTraverser} です。<br />
 * 
 * @author y-komori
 */
public class FileResourceTraverser implements ResourceTraverser {
    private static final String PROTOCOL = "file";

    /*
     * @see org.seasar.uruma.util.resource.ResourceTraverser#getProtocol()
     */
    public String getProtocol() {
        return PROTOCOL;
    }

    /*
     * @see
     * org.seasar.uruma.util.resource.ResourceTraverser#traverse(java.net.URL,
     * java.net.URL, org.seasar.uruma.util.resource.ResourceHandler,
     * org.seasar.uruma.util.resource.ResourceFilter)
     */
    public void traverse(final URL root, final URL origin,
            final ResourceHandler handler, final ResourceFilter filter) {
        if (origin == null) {
            throw new IllegalArgumentException("origin can't be null.");
        }
        if (handler == null) {
            throw new IllegalArgumentException("handler can't be null.");
        }

        String path = origin.getPath();
        if (isNotBlank(path)) {
            File file = new File(path);
            if (file.exists()) {
                if (file.isDirectory()) {
                    File rootDir = getRootDir(root, origin);
                    traverseFileSystem(rootDir, file, handler, filter);
                }
            }

        }
    }

    /*
     * @see
     * org.seasar.uruma.util.resource.ResourceTraverser#traverse(java.net.URL,
     * java.net.URL, org.seasar.uruma.util.resource.ResourceHandler)
     */
    public void traverse(final URL root, final URL origin,
            final ResourceHandler handler) {
        traverse(root, origin, handler, null);
    }

    protected void traverseFileSystem(final File baseDir, final File dir,
            final ResourceHandler handler, final ResourceFilter filter) {
        File[] files = dir.listFiles();
        String basePath = baseDir.getAbsolutePath().replace('\\', '/');

        for (int i = 0; i < files.length; ++i) {
            File file = files[i];
            if (file.isDirectory()) {
                traverseFileSystem(baseDir, file, handler, filter);
            } else {
                try {
                    int pos = -1;
                    if (baseDir != null) {
                        pos = baseDir.getCanonicalPath().length();
                    }
                    String filePath = file.getCanonicalPath();
                    String resourcePath = filePath.substring(pos + 1).replace(
                            '\\', '/');

                    if ((filter != null) && !filter.accept(resourcePath)) {
                        continue;
                    }

                    InputStream is = new FileInputStream(file);
                    try {
                        handler.handle(basePath, resourcePath, is);
                    } catch (Throwable ex) {
                    } finally {
                        if (is != null) {
                            is.close();
                        }
                    }
                } catch (IOException ex) {
                    throw new IORuntimeException(ex);
                }
            }
        }
    }

    protected File getRootDir(final URL root, final URL origin) {
        if (root != null) {
            String path = root.getPath();
            if (path != null) {
                File rootDir = new File(path);
                if (rootDir.exists()) {
                    return rootDir;
                }
            }
        }

        String path = origin.getPath();
        return new File(path);
    }

    private boolean isBlank(final String str) {
        if (str == null || str.length() == 0) {
            return true;
        }
        for (int i = 0; i < str.length(); i++) {
            if (!Character.isWhitespace(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    private boolean isNotBlank(final String str) {
        return !isBlank(str);
    }
}
